@@ -131,6 +131,7 @@ class BoostLibraryFile():
         name_parts = self.name.split('.')
         self.basename = name_parts[0]
         self.suffixes = name_parts[1:]
+        self.vers_raw = [x for x in self.suffixes if x.isdigit()]
         self.suffixes = [x for x in self.suffixes if not x.isdigit()]
         self.nvsuffix = '.'.join(self.suffixes)  # Used for detecting the library type
         self.nametags = self.basename.split('-')
@@ -138,6 +139,10 @@ class BoostLibraryFile():
         if self.mod_name.startswith('lib'):
             self.mod_name = self.mod_name[3:]
 
+        # Set library version if possible
+        if len(self.vers_raw) >= 2:
+            self.version_lib = '{}_{}'.format(self.vers_raw[0], self.vers_raw[1])
+
         # Detecting library type
         if self.nvsuffix in ['so', 'dll', 'dll.a', 'dll.lib', 'dylib']:
             self.static = False
@@ -152,6 +157,9 @@ class BoostLibraryFile():
 
         # Process tags
         tags = self.nametags[1:]
+        # Filter out the python version tag and fix modname
+        if self.is_python_lib():
+            tags = self.fix_python_name(tags)
         if not tags:
             return
 
@@ -226,6 +234,40 @@ class BoostLibraryFile():
     def is_python_lib(self) -> bool:
         return any([self.mod_name.startswith(x) for x in BoostLibraryFile.boost_python_libs])
 
+    def fix_python_name(self, tags: T.List[str]) -> T.List[str]:
+        # Handle the boost_python naming madeness.
+        # See https://github.com/mesonbuild/meson/issues/4788 for some distro
+        # specific naming variantions.
+        other_tags = []  # type: T.List[str]
+
+        # Split the current modname into the base name and the version
+        m_cur = BoostLibraryFile.reg_python_mod_split.match(self.mod_name)
+        cur_name = m_cur.group(1)
+        cur_vers = m_cur.group(2)
+
+        # Update the current version string if the new version string is longer
+        def update_vers(new_vers: str) -> None:
+            nonlocal cur_vers
+            new_vers = new_vers.replace('_', '')
+            new_vers = new_vers.replace('.', '')
+            if not new_vers.isdigit():
+                return
+            if len(new_vers) > len(cur_vers):
+                cur_vers = new_vers
+
+        for i in tags:
+            if i.startswith('py'):
+                update_vers(i[2:])
+            elif i.isdigit():
+                update_vers(i)
+            elif len(i) >= 3 and i[0].isdigit and i[2].isdigit() and i[1] == '.':
+                update_vers(i)
+            else:
+                other_tags += [i]
+
+        self.mod_name = cur_name + cur_vers
+        return other_tags
+
     def mod_name_matches(self, mod_name: str) -> bool:
         if self.mod_name == mod_name:
             return True
@@ -302,6 +344,7 @@ class BoostDependency(ExternalDependency):
         self.multithreading = kwargs.get('threading', 'multi') == 'multi'
 
         self.boost_root = None
+        self.explicit_static = 'static' in kwargs
 
         # Extract and validate modules
         self.modules = mesonlib.extract_as_list(kwargs, 'modules')  # type: T.List[str]
@@ -369,10 +412,19 @@ class BoostDependency(ExternalDependency):
                 break
 
     def run_check(self, inc_dirs: T.List[BoostIncludeDir], lib_dirs: T.List[Path]) -> bool:
+        mlog.debug('  - potential library dirs: {}'.format([x.as_posix() for x in lib_dirs]))
+        mlog.debug('  - potential include dirs: {}'.format([x.path.as_posix() for x in inc_dirs]))
+
         #   2. Find all boost libraries
         libs = []  # type: T.List[BoostLibraryFile]
         for i in lib_dirs:
-            libs += self.detect_libraries(i)
+            libs = self.detect_libraries(i)
+            if libs:
+                mlog.debug('  - found boost library dir: {}'.format(i))
+                # mlog.debug('  - raw library list:')
+                # for j in libs:
+                #     mlog.debug('    - {}'.format(j))
+                break
         libs = sorted(set(libs))
 
         modules = ['boost_' + x for x in self.modules]
@@ -380,9 +432,6 @@ class BoostDependency(ExternalDependency):
             mlog.debug('  - found boost {} include dir: {}'.format(inc.version, inc.path))
             f_libs = self.filter_libraries(libs, inc.version_lib)
 
-            # mlog.debug('  - raw library list:')
-            # for j in libs:
-            #     mlog.debug('    - {}'.format(j))
             mlog.debug('  - filtered library list:')
             for j in f_libs:
                 mlog.debug('    - {}'.format(j))
@@ -457,6 +506,19 @@ class BoostDependency(ExternalDependency):
         return [self._include_dir_from_version_header(x) for x in candidates]
 
     def detect_lib_dirs(self, root: Path) -> T.List[Path]:
+        # First check the system include paths. Only consider those within the
+        # given root path
+        system_dirs_t = self.clib_compiler.get_library_dirs(self.env)
+        system_dirs = [Path(x) for x in system_dirs_t]
+        system_dirs = [x.resolve() for x in system_dirs if x.exists()]
+        system_dirs = [x for x in system_dirs if mesonlib.path_is_in_root(x, root)]
+        system_dirs = list(mesonlib.OrderedSet(system_dirs))
+
+        if system_dirs:
+            return system_dirs
+
+        # No system include paths were found --> fall back to manually looking
+        # for library dirs in root
         dirs = []     # type: T.List[Path]
         subdirs = []  # type: T.List[Path]
         for i in root.iterdir():
@@ -468,7 +530,25 @@ class BoostDependency(ExternalDependency):
             for j in i.iterdir():
                 if j.is_dir() and j.name.endswith('-linux-gnu'):
                     subdirs += [j]
-        return dirs + subdirs
+
+        # Filter out paths that don't match the target arch to avoid finding
+        # the wrong libraries. See https://github.com/mesonbuild/meson/issues/7110
+        if not self.arch:
+            return dirs + subdirs
+
+        arch_list_32 = ['32', 'i386']
+        arch_list_64 = ['64']
+
+        raw_list = dirs + subdirs
+        no_arch = [x for x in raw_list if not any([y in x.name for y in arch_list_32 + arch_list_64])]
+
+        matching_arch = []  # type: T.List[Path]
+        if '32' in self.arch:
+            matching_arch = [x for x in raw_list if any([y in x.name for y in arch_list_32])]
+        elif '64' in self.arch:
+            matching_arch = [x for x in raw_list if any([y in x.name for y in arch_list_64])]
+
+        return sorted(matching_arch) + sorted(no_arch)
 
     def filter_libraries(self, libs: T.List[BoostLibraryFile], lib_vers: str) -> T.List[BoostLibraryFile]:
         # MSVC is very picky with the library tags
@@ -480,7 +560,7 @@ class BoostDependency(ExternalDependency):
         except (KeyError, IndexError, AttributeError):
             pass
 
-        libs = [x for x in libs if x.static == self.static]
+        libs = [x for x in libs if x.static == self.static or not self.explicit_static]
         libs = [x for x in libs if x.mt == self.multithreading]
         libs = [x for x in libs if x.version_matches(lib_vers)]
         libs = [x for x in libs if x.arch_matches(self.arch)]
@@ -525,6 +605,12 @@ class BoostDependency(ExternalDependency):
                 roots += paths
                 return roots  # Do not add system paths if BOOST_ROOT is present
 
+        # Add roots from system paths
+        inc_paths = [Path(x) for x in self.clib_compiler.get_default_include_dirs()]
+        inc_paths = [x.parent for x in inc_paths if x.exists()]
+        inc_paths = [x.resolve() for x in inc_paths]
+        roots += inc_paths
+
         # Add system paths
         if self.env.machines[self.for_machine].is_windows():
             # Where boost built from source actually installs it
@@ -546,8 +632,6 @@ class BoostDependency(ExternalDependency):
             roots += [x for x in candidates if x.name.lower().startswith('boost') and x.is_dir()]
         else:
             tmp = []  # type: T.List[Path]
-            # Add unix paths
-            tmp += [Path(x).parent for x in self.clib_compiler.get_default_include_dirs()]
 
             # Homebrew
             brew_boost = Path('/usr/local/Cellar/boost')
@@ -595,11 +679,8 @@ class BoostDependency(ExternalDependency):
         return BoostIncludeDir(hfile.parents[1], int(m.group(1)))
 
     def _extra_compile_args(self) -> T.List[str]:
-        args = []  # type: T.List[str]
-        args += ['-DBOOST_ALL_NO_LIB']  # Disable automatic linking
-        if not self.static:
-            args += ['-DBOOST_ALL_DYN_LINK']
-        return args
+        # BOOST_ALL_DYN_LINK should not be required with the known defines below
+        return ['-DBOOST_ALL_NO_LIB']  # Disable automatic linking
 
 
 # See https://www.boost.org/doc/libs/1_72_0/more/getting_started/unix-variants.html#library-naming
@@ -623,9 +704,9 @@ boost_arch_map = {
 ####      ---- BEGIN GENERATED ----      ####
 #                                           #
 # Generated with tools/boost_names.py:
-#  - boost version:   1.72.0
-#  - modules found:   158
-#  - libraries found: 42
+#  - boost version:   1.73.0
+#  - modules found:   159
+#  - libraries found: 43
 #
 
 class BoostLibrary():
@@ -648,16 +729,16 @@ class BoostModule():
 boost_libraries = {
     'boost_atomic': BoostLibrary(
         name='boost_atomic',
-        shared=[],
-        static=[],
+        shared=['-DBOOST_ATOMIC_DYN_LINK=1'],
+        static=['-DBOOST_ATOMIC_STATIC_LINK=1'],
         single=[],
         multi=[],
     ),
     'boost_chrono': BoostLibrary(
         name='boost_chrono',
-        shared=['-DBOOST_ALL_DYN_LINK=1'],
-        static=['-DBOOST_All_STATIC_LINK=1'],
-        single=[],
+        shared=['-DBOOST_CHRONO_DYN_LINK=1'],
+        static=['-DBOOST_CHRONO_STATIC_LINK=1'],
+        single=['-DBOOST_CHRONO_THREAD_DISABLED'],
         multi=[],
     ),
     'boost_container': BoostLibrary(
@@ -669,28 +750,28 @@ boost_libraries = {
     ),
     'boost_context': BoostLibrary(
         name='boost_context',
-        shared=[],
+        shared=['-DBOOST_CONTEXT_DYN_LINK=1'],
         static=[],
         single=[],
         multi=[],
     ),
     'boost_contract': BoostLibrary(
         name='boost_contract',
-        shared=[],
-        static=[],
-        single=[],
+        shared=['-DBOOST_CONTRACT_DYN_LINK'],
+        static=['-DBOOST_CONTRACT_STATIC_LINK'],
+        single=['-DBOOST_CONTRACT_DISABLE_THREADS'],
         multi=[],
     ),
     'boost_coroutine': BoostLibrary(
         name='boost_coroutine',
-        shared=[],
+        shared=['-DBOOST_COROUTINES_DYN_LINK=1'],
         static=[],
         single=[],
         multi=[],
     ),
     'boost_date_time': BoostLibrary(
         name='boost_date_time',
-        shared=[],
+        shared=['-DBOOST_DATE_TIME_DYN_LINK=1'],
         static=[],
         single=[],
         multi=[],
@@ -704,14 +785,14 @@ boost_libraries = {
     ),
     'boost_fiber': BoostLibrary(
         name='boost_fiber',
-        shared=[],
+        shared=['-DBOOST_FIBERS_DYN_LINK=1'],
         static=[],
         single=[],
         multi=[],
     ),
     'boost_fiber_numa': BoostLibrary(
         name='boost_fiber_numa',
-        shared=[],
+        shared=['-DBOOST_FIBERS_DYN_LINK=1'],
         static=[],
         single=[],
         multi=[],
@@ -725,84 +806,91 @@ boost_libraries = {
     ),
     'boost_graph': BoostLibrary(
         name='boost_graph',
-        shared=['-DBOOST_GRAPH_DYN_LINK=1'],
+        shared=[],
         static=[],
         single=[],
         multi=[],
     ),
     'boost_iostreams': BoostLibrary(
         name='boost_iostreams',
-        shared=['-DBOOST_IOSTREAMS_DYN_LINK=1', '-DBOOST_IOSTREAMS_DYN_LINK=1'],
+        shared=['-DBOOST_IOSTREAMS_DYN_LINK=1'],
         static=[],
         single=[],
         multi=[],
     ),
     'boost_locale': BoostLibrary(
         name='boost_locale',
-        shared=['-DBOOST_LOCALE_DYN_LINK=1'],
+        shared=[],
         static=[],
         single=[],
         multi=[],
     ),
     'boost_log': BoostLibrary(
         name='boost_log',
-        shared=['-DBOOST_LOG_DLL', '-DBOOST_LOG_DYN_LINK=1'],
+        shared=['-DBOOST_LOG_DYN_LINK=1'],
         static=[],
-        single=['BOOST_LOG_NO_THREADS'],
+        single=['-DBOOST_LOG_NO_THREADS'],
         multi=[],
     ),
     'boost_log_setup': BoostLibrary(
         name='boost_log_setup',
-        shared=['-DBOOST_LOG_DYN_LINK=1', '-DBOOST_LOG_SETUP_DLL', '-DBOOST_LOG_SETUP_DYN_LINK=1'],
+        shared=['-DBOOST_LOG_SETUP_DYN_LINK=1'],
         static=[],
-        single=['BOOST_LOG_NO_THREADS'],
+        single=['-DBOOST_LOG_NO_THREADS'],
         multi=[],
     ),
     'boost_math_c99': BoostLibrary(
         name='boost_math_c99',
-        shared=['-DBOOST_MATH_TR1_DYN_LINK=1'],
+        shared=[],
         static=[],
         single=[],
         multi=[],
     ),
     'boost_math_c99f': BoostLibrary(
         name='boost_math_c99f',
-        shared=['-DBOOST_MATH_TR1_DYN_LINK=1'],
+        shared=[],
         static=[],
         single=[],
         multi=[],
     ),
     'boost_math_c99l': BoostLibrary(
         name='boost_math_c99l',
-        shared=['-DBOOST_MATH_TR1_DYN_LINK=1'],
+        shared=[],
         static=[],
         single=[],
         multi=[],
     ),
     'boost_math_tr1': BoostLibrary(
         name='boost_math_tr1',
-        shared=['-DBOOST_MATH_TR1_DYN_LINK=1'],
+        shared=[],
         static=[],
         single=[],
         multi=[],
     ),
     'boost_math_tr1f': BoostLibrary(
         name='boost_math_tr1f',
-        shared=['-DBOOST_MATH_TR1_DYN_LINK=1'],
+        shared=[],
         static=[],
         single=[],
         multi=[],
     ),
     'boost_math_tr1l': BoostLibrary(
         name='boost_math_tr1l',
-        shared=['-DBOOST_MATH_TR1_DYN_LINK=1'],
+        shared=[],
         static=[],
         single=[],
         multi=[],
     ),
     'boost_mpi': BoostLibrary(
         name='boost_mpi',
-        shared=['-DBOOST_MPI_DYN_LINK=1'],
+        shared=[],
+        static=[],
+        single=[],
+        multi=[],
+    ),
+    'boost_nowide': BoostLibrary(
+        name='boost_nowide',
+        shared=['-DBOOST_NOWIDE_DYN_LINK=1'],
         static=[],
         single=[],
         multi=[],
@@ -823,63 +911,63 @@ boost_libraries = {
     ),
     'boost_random': BoostLibrary(
         name='boost_random',
-        shared=[],
+        shared=['-DBOOST_RANDOM_DYN_LINK'],
         static=[],
         single=[],
         multi=[],
     ),
     'boost_regex': BoostLibrary(
         name='boost_regex',
-        shared=['-DBOOST_REGEX_DYN_LINK=1'],
+        shared=[],
         static=[],
         single=[],
         multi=[],
     ),
     'boost_serialization': BoostLibrary(
         name='boost_serialization',
-        shared=['-DBOOST_SERIALIZATION_DYN_LINK=1'],
+        shared=[],
         static=[],
         single=[],
         multi=[],
     ),
     'boost_stacktrace_addr2line': BoostLibrary(
         name='boost_stacktrace_addr2line',
-        shared=['-DBOOST_STACKTRACE_DYN_LINK=1'],
+        shared=[],
         static=[],
         single=[],
         multi=[],
     ),
     'boost_stacktrace_backtrace': BoostLibrary(
         name='boost_stacktrace_backtrace',
-        shared=['-DBOOST_STACKTRACE_DYN_LINK=1'],
+        shared=[],
         static=[],
         single=[],
         multi=[],
     ),
     'boost_stacktrace_basic': BoostLibrary(
         name='boost_stacktrace_basic',
-        shared=['-DBOOST_STACKTRACE_DYN_LINK=1'],
+        shared=[],
         static=[],
         single=[],
         multi=[],
     ),
     'boost_stacktrace_noop': BoostLibrary(
         name='boost_stacktrace_noop',
-        shared=['-DBOOST_STACKTRACE_DYN_LINK=1'],
+        shared=[],
         static=[],
         single=[],
         multi=[],
     ),
     'boost_stacktrace_windbg': BoostLibrary(
         name='boost_stacktrace_windbg',
-        shared=['-DBOOST_STACKTRACE_DYN_LINK=1'],
+        shared=[],
         static=[],
         single=[],
         multi=[],
     ),
     'boost_stacktrace_windbg_cached': BoostLibrary(
         name='boost_stacktrace_windbg_cached',
-        shared=['-DBOOST_STACKTRACE_DYN_LINK=1'],
+        shared=[],
         static=[],
         single=[],
         multi=[],
@@ -900,8 +988,8 @@ boost_libraries = {
     ),
     'boost_thread': BoostLibrary(
         name='boost_thread',
-        shared=['-DBOOST_THREAD_USE_DLL=1'],
-        static=['-DBOOST_THREAD_USE_LIB=1'],
+        shared=['-DBOOST_THREAD_BUILD_DLL=1', '-DBOOST_THREAD_USE_DLL=1'],
+        static=['-DBOOST_THREAD_BUILD_LIB=1', '-DBOOST_THREAD_USE_LIB=1'],
         single=[],
         multi=[],
     ),
@@ -914,7 +1002,7 @@ boost_libraries = {
     ),
     'boost_type_erasure': BoostLibrary(
         name='boost_type_erasure',
-        shared=[],
+        shared=['-DBOOST_TYPE_ERASURE_DYN_LINK'],
         static=[],
         single=[],
         multi=[],
@@ -935,7 +1023,7 @@ boost_libraries = {
     ),
     'boost_wserialization': BoostLibrary(
         name='boost_wserialization',
-        shared=['-DBOOST_SERIALIZATION_DYN_LINK=1'],
+        shared=[],
         static=[],
         single=[],
         multi=[],
